/*
	fndecl.h   	10/15/86

	% public header for field functions

	This file contains field function struct definitions
	and various misc. routines.

	C-scape 3.1
	Copyright (c) 1986-1989, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 9/01/87 jmd 	added NO_PROTO option
	 6/01/88 jmd 	removed anachronisms
	 7/27/88 jmd 	removed string tools
	 8/22/88 jmd 	added field_funcs_struct, special funcs
	 9/16/88 jmd 	added _arg
	 9/17/88 jmd	added global error msg strings for easy changing
	 9/17/88 jmd	added std_ funcs
	10/06/88 jmd	added snum_fenter
	11/28/88 jmd	split into two files
	12/16/88 jmd	added valid_Float
	12/20/88 jmd	added click_funcs

	 4/06/89 jmd	Fixed std_fexit prototype
	 4/23/89 jdc	added pulldown_funcs
	 6/01/89 gam	added ocountry stuff
	 6/25/89 jmd	added OEXTERNs
	 7/15/89 ted	Converted '_func' prototypes from typedef to macro.
	 8/03/89 jmd	added nowrite_funcs
	 8/07/89 jmd	added time and date _funcs, std_NumValid
*/

/*** macros and #defines ***/

struct select_struct {
	int		number;
	char 	**list;
};

#include "fncntry.h"

#define select_build(select, choice_list)  \
	struct select_struct select = {\
			0,\
			choice_list\
	}

#define	select_number(select)  	((select).number)
#define select_choice(select)   ((select).list[(select).number])

/* ticker funcs stuff (obsolete) */

#define	tick_GetLen(t)				( *((int *) t))
#define	tick_SetLen(t, len)			( *((int *) t) = len)
#define	tick_GetOffset(t) 			( *((int *) (t+sizeof(int))) )
#define	tick_SetOffset(t, off)	 	( *((int *) (t+sizeof(int))) = off)
#define tick_GetString(t)			(t + (2 * sizeof(int)))


/*** Global error strings ***/

extern char   *fnstring_errmsg;		/* Used by string funcs (fnstring.c) */
extern char   *fnnum_errmsg;		/* Used by numeric funcs (fnnum.c) */

/*** Color of marked field ***/
extern byte	   fnmark_color;		/* used by mark_funcs (fnmark.c ) */

/*** Functions ***/

/*** special functions ***/

extern spc_func (spc_Embed);
extern spc_func (spc_EmbedTed);
extern spc_func (spc_Abort);

/*** std funcs ***/

extern void		std_fenter(_arg1(sed_type));
extern void		stdNoCur_fenter(_arg1(sed_type));
extern void		stdBigCur_fenter(_arg1(sed_type));
extern boolean	std_fexit(_arg1(sed_type));
extern void 	std_senter(_arg2(sed_type, int));

extern boolean 	std_NumValid(_arg2(sed_type, double));

/*** StrCommon funcs ***/

extern void StrCommon_fkey(_arg2(sed_type, boolean (*)(_arg1(int))));

/*** idle functions ***/

extern void 	idle_Clock(_arg1(int));

/*** Validation routines ***/

#define	VALID_SLEN	80		/* max length of validation string component */

extern void	   valid_Format(_arg3(sed_type, int, char *));

extern boolean valid_Double(_arg2(double, char *));
extern boolean valid_String(_arg2(char *, char *));

#define valid_Int(val, vstr)		valid_Double((double) (val), vstr)
#define valid_Long(val, vstr)		valid_Double((double) (val), vstr)
#define valid_Float(val, vstr)		valid_Double((double) (val), vstr)

/*** movement funcs ***/

extern boolean	inter_page(_arg2(sed_type, int));
extern boolean	inter_field(_arg2(sed_type, int));
extern boolean	inter_field_grid(_arg2(sed_type, int));
extern boolean 	special_key(_arg2(sed_type, int));

/*** field functions ***/

OEXTERN field_funcs_struct alpha_funcs;
OEXTERN field_funcs_struct bob_funcs;
OEXTERN field_funcs_struct char_funcs;
OEXTERN field_funcs_struct check_funcs;
OEXTERN field_funcs_struct click_funcs;
OEXTERN field_funcs_struct clong_funcs;
OEXTERN field_funcs_struct cmoney_funcs;
OEXTERN field_funcs_struct date_funcs;
OEXTERN field_funcs_struct digit_funcs;
OEXTERN field_funcs_struct double_funcs;
OEXTERN field_funcs_struct gmenu_funcs;
OEXTERN field_funcs_struct hex_funcs;
OEXTERN field_funcs_struct int_funcs;
OEXTERN field_funcs_struct list_funcs;
OEXTERN field_funcs_struct long_funcs;
OEXTERN field_funcs_struct mark_funcs;
OEXTERN field_funcs_struct menu_funcs;
OEXTERN field_funcs_struct money_funcs;
OEXTERN field_funcs_struct msys_funcs;
OEXTERN field_funcs_struct null_funcs;
OEXTERN field_funcs_struct nowrite_funcs;
OEXTERN field_funcs_struct pint_funcs;
OEXTERN field_funcs_struct plong_funcs;
OEXTERN field_funcs_struct range_funcs;
OEXTERN field_funcs_struct sdouble_funcs;
OEXTERN field_funcs_struct secure_funcs;
OEXTERN field_funcs_struct select_funcs;
OEXTERN field_funcs_struct sfloat_funcs;
OEXTERN field_funcs_struct sint_funcs;
OEXTERN field_funcs_struct slong_funcs;
OEXTERN field_funcs_struct string_funcs;
OEXTERN field_funcs_struct ticker_funcs;
OEXTERN field_funcs_struct time_funcs;
OEXTERN field_funcs_struct toggle_funcs;
OEXTERN field_funcs_struct togint_funcs;
OEXTERN field_funcs_struct xstring_funcs;
OEXTERN field_funcs_struct yesno_funcs;

