/*
  	framer.c

	% framer menu functions

	rules for user functions:

	they get passed two arguments  (a VOID * and an int).
	The VOID * is passed through msys_Go().
	The int is the value parameter from the frame_def struct.

	they return a positive value if processing is finished
	This value is passed up and returned from msys_Go
	0 acts like an ESCape was pressed.

	int user_func(VOID *, int);

	C-scape 3.1
	Copyright (c) 1988, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 8/30/88 jdc	re-created
	12/11/88 jmd	moved idata to ufunc _od
	12/18/88 jdc	fixed field NAME_MAXLEN bug

	03/08/89 jdc	remove field name stuff
	 4/22/89 jmd	changed border to bord
	 7/26/89 jmd	made frame_Lock a function
	 8/11/89 jmd	added call to ufunc_Open
*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "cscape.h"
#include "oaktag.h"				/* For IDs in msys.h */
#include "msys.h"

sed_type frame_create(frame_list, dir, bord, bk_clr, sel_clr, bd_clr, row, col, ffuncs, mfunc)
	struct frame_def frame_list[];
	bd_fptr bord;
	byte bk_clr, sel_clr, bd_clr;
	int dir, row, col;
	field_funcs_ptr ffuncs;
	mouhandler_fptr mfunc;
/*
	Create a menuing system object from a frame_list.
*/
{
	menu_type topmenu, popmenu;
	sed_type topsed;
	bob_type bob;
	int i, top;

	if ( frame_list[0].choice == NULL || (topmenu = menu_Open()) == NULL ) {
		return(NULL);
	}
	for (i = 1, top = 0, popmenu = NULL;; i++) {

		bob = NULL;
		if ( frame_list[i].choice == NULL ) {
			/* process field in top sed */

			if ( dir == MENU_VERTICAL && top > 0 ) {
				menu_Printf(topmenu, "\n");
			}

			if (popmenu != NULL) {
				menu_Flush(popmenu);
				if ((bob = sed_Open(popmenu)) == NULL) {
					return(NULL);
				}
				sed_SetColors(bob, bk_clr, bk_clr, sel_clr);
				sed_SetBorder(bob, bord);
				sed_SetBorderColor(bob, bd_clr);
				sed_SetLabel(bob, frame_list[top].value);
				sed_SetMouse(bob, mfunc);
				win_SetParentClip(bob, BOB_INDEPENDENT);
			}
			else if ( (bob = ufunc_Open(frame_list[top].frame_func, 
					frame_list[top].value)) == NULL ) {

				return(NULL);
			}
			menu_Printf(topmenu, "@fb[ %s ]", NULL, ffuncs, bob,
				frame_list[top].choice);

			if ( frame_list[i+1].choice == NULL ) {
				/* done parsing frame_list */
				break;
			}
			else {
				i++;
				top = i;
			}
			popmenu = NULL;
		}
		else {
			/* process field in pop sed */
			if ( popmenu ==	NULL ) {
				if ( (popmenu = menu_Open()) == NULL ) {
					return(NULL);
				}
			}				
			else {
				menu_Printf(popmenu, "\n");
			}

			if ( (bob = ufunc_Open(frame_list[i].frame_func, 
				frame_list[i].value)) == NULL ) {

				return(NULL);
			}
			menu_Printf(popmenu, "@fb[ %s ]", NULL, ffuncs, bob,
				frame_list[i].choice);
		}
	}			
	menu_Flush(topmenu);

	if ((topsed = sed_Open(topmenu)) == NULL) {
		return(NULL);
	}
	sed_SetColors(topsed, bk_clr, bk_clr, sel_clr);
	sed_SetWidth(topsed, disp_GetWidth());
	sed_SetPosition(topsed, row, col);
	sed_SetLabel(topsed, frame_list[0].value);
	sed_SetMouse(topsed, mfunc);

	return(topsed);
}

boolean frame_do(sed, msg, family, member, indata)
	sed_type sed;
	int msg;
	int family, member;
	VOID *indata;
/*
*/
{
	int code[3];

	code[0] = family;
	code[1] = member;
	code[2] = -1;

	return(msys_do(sed, msg, code, indata));
}

boolean frame_Lock(sed, family, member, attr)
	sed_type sed;
	int family;
	int member;
	byte attr;
/*
	"lock" a menu choice in a framer menu.
*/
{
	return(frame_do(sed, MSYS_LOCK, family, member, (VOID *) &attr));
}


int frame_Go(sed, chr, data)
	sed_type sed;
	int chr;
	VOID *data;
{
	int choice;

	if (chr == ' ') {
		choice = -1;
	}
	else if ((choice = sed_SearchMerge(sed, (char)chr)) == -1) {
		return(FALSE);
	}

	return(msys_Go(sed, choice, -1, -1, data));
}
