/*
  	puview.c

	% pop_View

	Popup file viewing routine.

	C-scape 3.1
	Copyright (c) 1986, 1987, 1988 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 8/06/87 jmd 	added missing help function
	 6/02/88 jdc	converted to new textbuf stuff
	 8/23/88 jmd	fixed SetTB, removed attributes
	 9/16/88 jmd	updated for 3.0
	11/05/88 jmd	removed menu_Close

	 4/22/89 jmd	changed border to bord
	 6/12/89 jmd	removed kb_Clear
	 9/07/89 jmd	fixed default size trouble
*/

#include <stdio.h>
#include <string.h>

#include "cscape.h"

#include "scancode.h"
#include "popdecl.h"
#include "helpdecl.h"

void pop_View(title, text, row, col, height, width, color, label, bord)
	char *title;
	char *text;
	int  row, col;
	int  height, width;
	byte color;
	int  label;
	bd_fptr bord;
/*
	Places text in a window.

	Allows the text to be scrolled around.

	Exits when ESC is pressed.

	standard use of row, col etc...
*/
{
	sed_type   sed;
	menu_type  menu;
	int	exit = FALSE;

	if ((menu = menu_Open()) == NULL) {
		return;
	}
	
	if ((sed = sed_Open(menu)) == NULL) {
		menu_Destroy(menu);
		return;
	}

	/* put the text into the sed */
	sed_RewindTB(sed);
	if (!sed_SetTB(sed, text, strlen(text))) {
		sed_Close(sed);
		return;
	}

	sed_SetColors(sed, color, color, color);
	sed_SetBorder(sed, bord);
	sed_SetLabel(sed, label);
	sed_SetCursorType(sed, CURSOR_NONE);	/* turn off the cursor */

	/* pre-adjust for pop_Parms */
	height = (height < 0) ? DEFAULT_HEIGHT : height;
	width  = (width  < 0) ? DEFAULT_WIDTH  : width;

	pop_SetParms(sed, row, col, height, width, title);

	/* put up the sed */
	sed_Repaint(sed);
	sed_BorderPrompt(sed, "Press Escape");

	while(!exit) {
		switch(kb_Read()) {
			case FN1:
				help_Show(sed_GetLabel(sed), 0);
				break;
			case INS:
				break;
			case HOME:
				sed_ScrollUp(sed, sed_GetYoffset(sed)); 
				break;
			case END:
				sed_ScrollDown(sed, sed_GetMenuHeight(sed)); 
				break;
			case UP:
				sed_ScrollUp(sed, 1);
				break;
			case DOWN:
				sed_ScrollDown(sed, 1);
				break;
			case LEFT:
				sed_ScrollLeft(sed, 5);
				break;
			case RIGHT:
				sed_ScrollRight(sed, 5);
				break;
			case CTRL_LEFT:
				sed_ScrollLeft(sed, sed_GetXoffset(sed));
				break;
			case CTRL_RIGHT:
				sed_ScrollRight(sed, sed_GetMenuWidth(sed));
				break;
			case PGUP:
				sed_PageUp(sed);
				break;
			case PGDN:
				sed_PageDown(sed);
				break;
			case ESC:
				exit = TRUE;
				break;
			default:
				break;
		}
	}

	sed_Close(sed);
}
	
