/*
	sledopen.c

	% sled_ReallyOpen, sled_remapper

	C-scape 3.1
	Copyright (c) 1988-1989, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 7/22/89 jdc	removed from sledwin.c
	 7/29/89 jmd	added casts for blank
	 8/04/89 gam	changed #include <sledwin.h> to sled.h
	 8/05/89 gam	added protection/marking of rows and columns
	 8/13/89 jmd	revamped open/init mechanism
	 8/24/89 gam	@fp[] in menu_Printf now works in sleds
	 8/26/89 jmd	made torf an int
*/

#include "sed.h"
#include "sldwinod.h"
#include "sadecl.h"
#include "sled.h"
#include "sledprot.h"

sed_type sled_ReallyOpen(menu, hgt, spec_func, class)
	menu_type menu;
	int hgt;
	spc_fptr spec_func;
	class_fptr class;
/*
	Create fields for the sled from the one line menu.
	Open a sed and initialize it.
*/
{
	sed_type sed;
	int 	 i, row, fld;

	/* find the first (and only) row */
	for (row = 0; (fld = menu_GetGRow(menu, row)) <= 0; row++) {
		;
	}

	/* Don't forget that the fldno in the fld grid is the real fldno plus 1 */
	fld--;

	/* create hgt rows from the one row */
	do {
		for (i = 1; i < hgt; i++) {
			if (menu_CopyField(menu, row + i, menu_GetFieldCol(menu, fld), menu, fld) < 0) {
				return(NULL);
			}
		}
	} while ((fld = menu_GetFieldRight(menu, fld)) >= 0);

	/* open the sed */
	if ((sed = sed_ReallyOpen(menu, class)) == NULL)  {
		return(NULL);
	}

	sled_Init(sed, hgt, spec_func);

	return(sed);
}

boolean sled_Init(sed, hgt, spec_func)
	sed_type sed;
	int hgt;
	spc_fptr spec_func;
/*
	Initialize a sled.
*/
{
	int 	 i, col, fld, varsize, max_varsize = 0;
	xarray 	 xa;
	VOID 	*var;
	char	*blank;
	sarray_type sa;
	iarray	 ca;
	int		 flag = FALSE;
	menu_type menu;
	int		 torf;

	menu = sed_GetMenu(sed);

	/* The column arrays are an xarray of sarrays */
	if ((xa = xa_Open(1)) == NULL) {
		goto QUIT_1;
	}

	/* find the first (and only) row */
	fld = sed_GetGridField(sed, 0, 0);
	col = 0;

	/* make iarray for column protection and initialize it. */
	if ((ca = ia_Open(1)) == NULL) {
		goto QUIT_2;
	}

	/* create hgt rows from the one row */
	do {
		/* create the Column Arrays (sarrays) checking for NULL varsize */
		if ((varsize = menu_GetVarSize(menu, fld)) == 0) {
			sa = NULL;
		}
		else {
			if ((sa = sa_Open(varsize, SLED_BSIZE, SLED_ASIZE)) == NULL) {
				goto QUIT_3;
			}
		}
		/* Put the sarray into the xarray */
		xa_Put(xa, col, (VOID *)sa);

		/* find the largest variable to allocate blank field var */
		max_varsize = (max_varsize < varsize) ? varsize : max_varsize;

		/* Set column protection flag */
		torf = (int) sed_IsProtectedField(sed, fld);
		ia_Put(ca, col, torf);

		/* Next column, please */
		col++;

	} while ((fld = menu_GetFieldRight(menu, fld)) >= 0);

	/* make sarray for mark/protect */
	if ((sa = sa_Open(sizeof(int), SLED_BSIZE, SLED_ASIZE)) == NULL) {
		goto QUIT_4;
	}
	xa_Put(xa, col, (VOID *)sa);

	/* Initialize the row special flag to 0 */
	for (i = 0; i < hgt; i++) {
		sa_Put(sa, (long) i, &flag);
	}

	/* allocate the blank field memory */
	if ((blank = (char *) omalloc(CSA_SLEDBLANK, max_varsize)) == NULL) {
		goto QUIT_5;
	}

	/* set the col protect/mark flags */
	sled_setcflags(sed, ca);

	if (spec_func != FNULL) {
		sed_SetSpecial(sed, spec_func);
	}

	/* initialize the blank data */
	memset(blank, 0, max_varsize);
	sled_SetBlank(sed, (VOID *) blank);

	/* create one empty row */
	for (i = 0; i < col; i++) {
		var = sa_Put((sarray_type)xa_Get(xa, i), 0L, sled_GetBlank(sed));
		sed_SetVar(sed, i, var);
	}

	sled_SetOffset(sed, 0);
	sled_SetColSize(sed, 1);
	sled_SetXa(sed, xa);
	sled_SetWidth(sed, col);

	sled_remapper(sed, 0);

	return(TRUE);

	/* These statements clean up if not enough memory for everything */
QUIT_5:
	/* The mark/protect sarray gets closed in the QUIT_2 part. */

QUIT_4:
QUIT_3:
	for ( i = 0; i <= col; i++ ) {
		sa_Close((sarray_type)xa_Get(xa, i));
	}
	ia_Close(ca);

QUIT_2:
	xa_Close(xa);

QUIT_1:
	return(FALSE);
}

boolean sled_remapper(sed, offset)
	sed_type sed;
	int offset;
{
	boolean	  protect, marked;
	int 	  bot, fld, row, col, i;
	menu_type menu;

	menu = sed_GetMenu(sed);

	if (offset < 0) {
		if (sled_GetOffset(sed) == 0) {
			return(FALSE);
		}
		offset = (sled_GetOffset(sed) < -offset) ? -sled_GetOffset(sed) : offset;
	}
	else if (offset > 0) {
		if ((bot = sled_GetColSize(sed) - (sled_GetHeight(sed) + sled_GetOffset(sed))) <= 0) {
			return(FALSE);
		}
		offset = (bot < offset) ? bot : offset;
	}
	sled_SetOffset(sed, sled_GetOffset(sed) + offset);

	for (i = 0, row = sled_GetOffset(sed); i < menu_GetRowCount(menu); i++) {
		if ((fld = menu_GetGRow(menu, i) - 1) < 0) {
			continue;
		}
		protect = sled_IsProtectedRow(sed, row);
		marked  = sled_IsMarkedRow(sed, row);
		/* loop each row */
		col = 0;
		do {
			/* loop each col */
			if (row < sled_GetColSize(sed)) {
				/* row exists */
				menu_SetVar(menu, fld, sa_Get((sarray_type)xa_Get(sled_GetXa(sed), col), (long)row));
				menu_ProtectField(menu, fld, protect | sled_IsProtectedCol(sed, col));
				if (marked | sled_IsMarkedCol(sed, col)) {
					menu_MarkField(menu, fld, sled_GetRegAttr(sed), sled_GetSelAttr(sed));
				}
				else {
					field_SetMarked(sed_GetField(sed, fld), FALSE);
				}
			}
			else {
				/* row doesn't exist */
				menu_SetVar(menu, fld, sled_GetBlank(sed));
				menu_ProtectField(menu, fld, TRUE);
			}
			col++;

		} while ((fld = menu_GetFieldRight(menu, fld)) >= 0);
		row++;
	}
	/* make sure height is correct */
	sled_SetHeight(sed, row - sled_GetOffset(sed));

 	/* Send Scroll message to the border */
	sed_SendBorderMsg(sed, BDM_SCROLL, NULL, NULL);

	return(TRUE);
}
