/*
	sledwin.c

	% sledwin_Class: The sled window class

	C-scape 3.1
	Copyright (c) 1988-1989, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 2/07/89 jmd	initialized sleddata to avoid warning
	 3/24/89 jmd	added sed_ macros
	 3/29/89 jmd	converted seds to 'pure' objects
	 4/11/89 jmd	fixed field blanking (this fixes graphics mode bug)
					fixed GoHome in ScrollReq
	04/19/89 jdc	changed sled_Open() to sled_ReallyOpen()
	04/28/89 jdc	removed protection from sled_remapper()
	05/23/89 jdc	put protection back (bob grid movement requires it)
					added failure checks to sled_ReallyOpen()
					added support for varsize == 0
	 7/22/89 jdc	moved sled_ReallyOpen and sled_remapper to sledopen.c
	 8/04/89 gam	changed #include <sledwin.h> to sled.h
	 8/05/89 gam	added protection/marking for rows and columns
	 8/06/89 gam	added sled_getrowflags function for Microsoft
	 8/10/89 gam	added remapper check to sled_SetColVar().
	 8/21/89 jdc	broke out close code into sledwin_Close (LNF)
	 8/23/89 jmd	added tests for menu == NULL
	 9/12/89 gam	added scroll message to sled_remapper
*/

#include "sed.h"
#include "sldwinod.h"
#include "sadecl.h"
#include "sled.h"
#include "sledprot.h"

int sledwin_Class(objdata, msg, indata, outdata)
	VOID *objdata;			/* object instance data pointer */
	int msg;				/* message */
	VOID *indata;			/* message input data */
	VOID *outdata;			/* message output data */
/*
	Dispatch function for sed windows.
*/
{
	sledwin_od		 *sledwd;
	inposdata_struct *ipd;
	ofont_type 		  font;
	sed_type		  sed;
	menu_type 		  menu;
	int				  row, col, fld, i, dif;

	sledwd = (sledwin_od *) objdata;

	switch (msg) {

	case OBJM_LOAD:
		/* sedwin_Class handle handles request linking */
		if (sedwin_DoRaw(&(sledwd->sedwd), msg, indata, outdata)) {
			if (sled_Init(sledod_GetSelf(sledwd), 1, FNULL)) {
				break;
			}
		}
		return(FALSE);

	case OBJM_GETDATASIZE:
		((ogds_struct *) outdata)->odsize = sizeof(sledwin_od);
		((ogds_struct *) outdata)->xdsize = sizeof(sled_xd);
		((ogds_struct *) outdata)->id = ID_SLEDWIN;
		break;

	case OBJM_WHO:
		/* Identify ourselves */
		if (*((int *) indata) == ID_SLEDWIN) {
			return(TRUE);
		}
		return(sedwin_DoRaw(&(sledwd->sedwd), msg, indata, outdata));

	case WINM_SETSIZE:
		sed = sledod_GetSelf(sledwd);

		/* remember old size */
		row = sed_GetHeight(sed);

		/* resize sed */
		sedwin_DoRaw(&(sledwd->sedwd), msg, indata, outdata);

		if ((menu = sed_GetMenu(sed)) != NULL) {

			col = 0;
			if ((dif = sed_GetHeight(sed) - row) > 0) {

				/* adjust the sed yoffset if possible */
				if ((col = sed_GetYoffset(sed)) > 0) {
					col = (col < dif) ? col : dif;
					sed_ScrollUp(sed, col);
					dif -= col;
				}
				/* adjust the sled offset if possible */
				if ((col = sled_GetOffset(sed)) > 0) {
					col = (col < dif) ? col : dif;
				}

				row = menu_GetHeight(menu);
				/* add fields if necessary */
				for (fld = 0; fld >= 0; fld = menu_GetFieldRight(menu, fld)) {
					for (i = sed_GetHeight(sed) - 1; i >= row; i--) {
						menu_CopyField(menu, i, menu_GetFieldCol(menu, fld), menu, fld);
					}
				}
				sled_remapper(sed, 0);
				if (col != 0) {
					sed_DoSexits(sed);
				}
				sled_remapper(sed, -col);
			}
			if (win_IsEmployed(sed)) {
				sed_Repaint(sed);
			}
			for (; col > 0; col--) {
				sed_DownField(sed);
			}
		}
		break;

	case WINM_SCROLLREQ:
		/* A request to scroll the sled */
		sed = sledod_GetSelf(sledwd);
		row = ((opoint *)indata)->y / win_GetFontHeight(sed);
		col = ((opoint *)indata)->x / win_GetFontWidth(sed);

		if (col < 0) {
			sed_ScrollLeft(sed, -col);
		}
		else if (col > 0) {
			sed_ScrollRight(sed, col);
		}
		if (row != 0 && sled_Remap(sed, row)) {
			/* the field should appear to be newly entered */
			sed_GoHome(sed);
			sed_DoFieldFenter(sed, sed_GetFieldNo(sed));
			if (row < 0) {
				sed_DownField(sed);
			}
			else {
				sed_UpField(sed);
			}
		}
		break;

	case WINM_GETINPOS:
		/* get the menu size relative to the sed
		   so that borders can compute scroll lights
		*/
		ipd = (inposdata_struct *)outdata;
		font = win_GetFont(ipd->win);

		sed = sledod_GetSelf(sledwd);

		if (sed_GetMenu(sed) != NULL) {
			/* Xoffset same as seds */
			ipd->inbox.xmin = - (sed_GetXoffset(sed) * ofont_GetWidth(font));
			ipd->inbox.xmax = ipd->inbox.xmin +
							(sed_GetMenuVWidth(sed) * ofont_GetWidth(font));

			/* but not Yoffset */
			ipd->inbox.ymin = - ((sed_GetYoffset(sed) + sled_GetOffset(sed)) * ofont_GetHeight(font));
			ipd->inbox.ymax = (sled_GetColSize(sed) +
				(sed_GetMenuHeight(sed) - sled_GetHeight(sed)))
				* ofont_GetHeight(font);
			ipd->inbox.ymax += ipd->inbox.ymin;
		}
		break;

	case WINM_PAINT:
		/* Set field blanking flag to prevent painting of
		   fields past the current last row of the sled
		*/	
		sed = sledod_GetSelf(sledwd);

		sledwd->sedwd.blank = sled_GetColSize(sed) - sled_GetOffset(sed) 
							+ field_GetRow(sed_GetField(sed, 0));

		/* pass message up to sed superclass */
		sedwin_DoRaw(&(sledwd->sedwd), msg, indata, outdata);

		/* reset blanking flag */
		sledwd->sedwd.blank = -1;
		break;

	case OBJM_CLOSE:
		sledwin_Close(sledod_GetSelf(sledwd));

		/* No Break; fall through to default */

	default:
		return(sedwin_DoRaw(&(sledwd->sedwd), msg, indata, outdata));
	}
	return(TRUE);
}

/* Auxillary Functions */

void sledwin_Close(sed)
	sed_type sed;
{
	sarray_type		  sa;
	int i;

	/* close the column arrays */
	for (i = 0; (sa = (sarray_type)xa_Get(sled_GetXa(sed), i)) != NULL; i++) {
		sa_Close(sa);
	}
	/* close the xarray of column arrays */
	xa_Close(sled_GetXa(sed));

	/* close the column marking/protection array */
	ia_Close(sled_getcflags(sed));

	/* close the blank field */
	ofree(CSA_SLEDBLANK, sled_GetBlank(sed));
}

boolean sled_Remap(sed, offset)
	sed_type sed;
	int offset;
/*
*/
{
	boolean ret = FALSE;
	boolean stuck = FALSE;
	int 	row, scrl, oldoffset;

	oldoffset = offset;

	/* The scrolling phrases need this value of row. */
	row = sled_GetRow(sed);

	if (offset < 0) {
		/* Make sure scrolling to an unprotected row */
		while(sled_IsProtectedRow(sed,row+offset)) {
			if (stuck) {
				if (offset ==0) {
					return(FALSE);
				}
				else {
					offset++;
				}
			}
			else { /* Not stuck */
				if (row + offset < 0) {
					stuck = TRUE;
				}
				else {
					offset--;
				}
			}
		}

		/* Toss out the old value of row.  We don't need it any more. */
		row = sed_GetYoffset(sed);
		sed_ScrollUp(sed, sled_GetHeight(sed));
		row -= sed_GetYoffset(sed);
		offset += row;
		sed_DoSexits(sed);
	}
	else if (offset > 0) {
		scrl = sled_GetColSize(sed);
		while (sled_IsProtectedRow(sed,row+offset)) {
			if (stuck) {
				if (offset == scrl) {
					return(FALSE);
				}
				else {
					offset--;
				}
			}
			else { /* Not stuck */
				if (row + offset > scrl) {
					stuck = TRUE;
				}
				else {
					offset++;
				}
			}
		}

		/* Toss out old value of scrl.  No longer need it. */
		scrl = sled_GetColSize(sed) - sled_GetOffset(sed) - sed_GetYoffset(sed) - sled_GetHeight(sed);
		scrl = (scrl < 0) ? 0 : scrl;
		offset = (scrl > offset) ? offset : scrl;
		/* Toss out old value of row.  Don't need it anymore. */
		row = sed_GetYoffset(sed);
		sed_ScrollDown(sed, offset);
		row = sed_GetYoffset(sed) - row;
		if ((offset -= row) == 0) {
			return(FALSE);
		}
		sed_DoSexits(sed);
	}
	if ((ret = sled_remapper(sed, offset)) == TRUE) {
		/* Move out of Protected Row in the original direction */
		if (sled_IsProtectedRow(sed, sled_GetRow(sed))) {
			if (oldoffset < 0) {
				sed_UpField(sed);
			}
			else {
				sed_DownField(sed);
			}
		}
		sed_RepaintFields(sed);
	}

	return(ret);
}

VOID *sled_GetColVar(sed, col, row)
	sed_type sed;
	int col, row;
/*
*/
{
	return(sa_Get((sarray_type) xa_Get(sled_GetXa(sed), col), (long)row));
}

void sled_SetColVar(sed, col, row, data)
	sed_type sed;
	int col, row;
	VOID *data;
/*
*/
{
	int i;
	boolean flag;

	if (row >= sled_GetColSize(sed)) {
		for (i = sled_GetColSize(sed); i <= row; i++) {
		/* Initialize the protect and mark flags */
			flag = FALSE;
			sled_setrowspecial(sed, i, flag);
		}
		sled_SetColSize(sed, row + 1);
		sed_SendBorderMsg(sed, BDM_SCROLL, NULL, NULL);
	}

	sa_Put((sarray_type) xa_Get(sled_GetXa(sed), col), (long)row, data);

	/* We only call remapper if row is currently "in" the sed */
	if (row >= sled_GetOffset(sed) && row < (sled_GetOffset(sed) + sled_GetHeight(sed))) {
		sled_remapper(sed, 0);
 	}
}


int sled_getrowspecial(sed, row)
	sed_type	sed;
	int			row;
{
	int			*temp;

	temp = (int *)sa_Get(xa_Get(sled_GetXa(sed), sled_GetWidth(sed)), (long)(row));

	if (temp == NULL) {
		return(0);
	}
	else {
		return(*temp);
	}
}


