/*
  	slug.c

	% slug menu routines

	rules for user functions:

	they get passed two arguments  (a VOID * and an int).
	The VOID * is passed through msys_Go().
	The int is the value parameter from the slug_list struct.

	they return a positive value if processing is finished
	This value is passed up and returned from msys_Go
	0 acts like an ESCape was pressed.

	int user_func(VOID *, int);

	C-scape 3.1
	Copyright (c) 1988, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 8/30/88 jdc	re-created
	12/11/88 jmd	moved idata to ufunc _od

	 4/22/89 jmd	changed border to bord
	 8/11/89 jmd	added call to ufunc_Open
*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "cscape.h"
#include "oaktag.h"				/* For IDs in msys.h */
#include "msys.h"

#define	SLUG_MIN_WIDTH	20

sed_type slug_create(slug_def, dir, bord, bkg, sel, bdr, row, col, ffuncs, mfunc)
	struct slug_list slug_def[];
	bd_fptr bord;
	byte bkg, sel, bdr;
	int dir, row, col;
	field_funcs_ptr ffuncs;
	mouhandler_fptr mfunc;
/*
	Create a menuing system object from a slug_list.

	This function gets called recursively.
*/
{
	menu_type menu;
	sed_type sed;
	bob_type bob;
	int i, dif;

	if ((menu = menu_Open()) == NULL) {
		return(NULL);
	}

	for ( i = 0, bob = NULL; slug_def[i].choice != NULL ; i++, bob = NULL ) {

		if ( dir == MENU_VERTICAL && i > 0 ) {

			menu_Printf(menu, "\n");
		}
		if (slug_def[i].child != NULL) {			/* make the child */

			if ((bob = slug_create(slug_def[i].child, dir, bord, bkg, 
				sel, bdr, row, col, ffuncs, mfunc)) == NULL) {

				return(NULL);
			}
			win_SetParentClip(bob, BOB_INDEPENDENT);
		}
		else if ((bob = ufunc_Open(slug_def[i].slug_func, slug_def[i].value)) == NULL ) {

			return(NULL);
		}		

		menu_Printf(menu, "@fbd[ %s ]", NULL, ffuncs, bob,
			slug_def[i].message, slug_def[i].choice);
	}
	menu_Flush(menu);

	if ((sed = sed_Open(menu)) == NULL) {
		return(NULL);
	}
	sed_SetColors(sed, bkg, bkg, sel);
	sed_SetBorder(sed, bord);
	sed_SetBorderColor(sed, bdr);
	sed_SetBorderTitle(sed, slug_def[i].message);
	sed_SetLabel(sed, slug_def[i].value);
	sed_SetPosition(sed, row, col);
	sed_SetMouse(sed, mfunc);
	if ( dir == MENU_VERTICAL ) {
		if ( sed_GetWidth(sed) < SLUG_MIN_WIDTH ) {
			sed_SetWidth(sed, SLUG_MIN_WIDTH);
		}
		if ( (dif = sed_GetBorderHeight(sed) + row - disp_GetHeight()) > 0 ) {
			sed_SetHeight(sed, sed_GetHeight(sed) - dif);
		}
	}
	else {
		sed_SetWidth(sed, disp_GetWidth() - col);
	}

	return(sed);
}

int slug_Go(sed, start, row, col, data)
	sed_type sed;
	int start, row, col;
	VOID *data;
/*
*/
{
	int ret;

	msys_Repaint(sed, row, col);
	ret = msys_Go(sed, start, row, col, data);
	sed_Pop(sed);

	return(ret);
}
