/*
	teddecl.h  4/2/88 jdc

	% public header for textbuffer editting

	C-scape 3.1
	Copyright (c) 1988 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 9/15/87 jmd 	added _arg, extern
	11/28/88 jmd	reorganized headers, added sed_GetCRow, sed_GetCCol.

	02/02/89 jdc	fixed move_method typedef
	 3/31/89 jmd	Cleaned up some macros
	 7/15/89 ted	Converted '_func' prototypes from typedef to macro.
*/

#define TED_BACKWARD	0x00
#define TED_FORWARD		0x01
#define TED_AFTER		0x02
#define TED_BEFORE		0x04

#define TED_NOMATCH		0
#define TED_MATCH		1
#define TED_MORE		2

#define TED_NOREFRESH	0
#define TED_REFRESH		1

#define TED_NOTABJUMP	0
#define TED_TABJUMP		1

#define TED_HARD		0
#define TED_SOFT		1
#define TED_LEN			2

#define B_CUT			1
#define B_COPY			2
#define B_ATTR			4
#define B_PAD			8

/* ted_funcs, in fnted.c */
extern void 	ted_fenter(_arg1(sed_type));
extern void 	ted_fkey(_arg1(sed_type));
extern void  	ted_senter(_arg2(sed_type, int));
extern void 	ted_sexit(_arg2(sed_type, int));

OEXTERN field_funcs_struct ted_funcs;

extern spc_func (spc_EmbedTed);

extern _move_func (ted_Past);
extern _move_func (ted_Follow);

/*	character functions */
extern boolean	ted_AddChar(_arg2(sed_type, char));
extern boolean	ted_AddString(_arg3(sed_type, char*, unsigned int));
extern char		ted_DeleteString(_arg4(sed_type, int, int, unsigned int));
extern unsigned int	ted_GetString(_arg3(sed_type, char*, unsigned int));

/*	cursor movement functions */
extern void		ted_StartWorking(_arg1(sed_type));
extern boolean	ted_LeftChar(_arg2(sed_type, int));
extern boolean	ted_RightChar(_arg2(sed_type, int));
extern void		ted_GetPosition(_arg3(sed_type, int*, int*));
extern boolean	ted_GoEnd(_arg1(sed_type));
extern void		ted_PageUp(_arg1(sed_type));
extern void		ted_PageDown(_arg1(sed_type));
extern void		ted_GoBottom(_arg1(sed_type));
extern boolean	ted_FindWord(_arg2(sed_type, int));
extern boolean	ted_Search(_arg3(sed_type, char*, int));
extern long		ted_GetCursor(_arg1(sed_type));
extern boolean	ted_GotoCursor(_arg2(sed_type, long));

/*	block manipulation functions */
extern int 		ted_GetMark(_arg1(sed_type));
extern void 	ted_SetMark(_arg2(sed_type, int));
extern boolean	ted_copy(_arg4(sed_type, menu_type, byte, int));
extern boolean	ted_insert(_arg2(sed_type, menu_type));

/*	others */
extern void		ted_reset(_arg3(sed_type, int, int));
extern boolean	ted_Scroll(_arg1(sed_type));
extern void		ted_SetCursor(_arg3(sed_type, int, int));
extern long		ted_ReadFile(_arg2(sed_type, FILE*));
extern long		ted_WriteFile(_arg3(sed_type, FILE*, int));
extern int 		ted_line(_arg2(sed_type, int));
extern void		ted_SetMoveMethod(_arg2(sed_type, move_method));

/** Macros **/

#define	sed_GetCRow(sed)		((sed_getxd(sed))->row)
#define sed_GetCCol(sed)		((sed_getxd(sed))->col)

#define ted_GotoPosition(sed, row, col)  (*((sed_getxd(sed))->move))(sed, row, col)

#define ted_AddRow(sed)			ted_AddChar(sed, '\n')
#define ted_DownChar(sed)  		ted_GotoPosition(sed, sed_GetCRow(sed) + 1, sed_GetCCol(sed))
#define ted_UpChar(sed)  		ted_GotoPosition(sed, sed_GetCRow(sed) - 1, sed_GetCCol(sed))
#define ted_GoHome(sed)			ted_GotoPosition(sed, sed_GetCRow(sed), 0)
#define ted_GoTop(sed)			ted_GotoPosition(sed, 0, 0)
#define ted_LeftWord(sed)		ted_FindWord(sed, TED_BACKWARD)
#define ted_RightWord(sed)		ted_FindWord(sed, TED_FORWARD)

#define ted_DeleteChar(sed) 	ted_DeleteString(sed, sed_GetCRow(sed), sed_GetCCol(sed), 1)

/*	buf is expected to be a menu_type */
#define ted_BlockCut(sed, buf)		ted_copy(sed, buf, 0x00, B_CUT + B_COPY)
#define ted_BlockCopy(sed, buf)		ted_copy(sed, buf, 0x00, B_COPY)
#define ted_BlockCopyPad(sed, buf)	ted_copy(sed, buf, 0x00, B_PAD + B_COPY)
#define ted_BlockDelete(sed)		ted_copy(sed, NULL, 0x00, B_CUT)
#define ted_BlockAttr(sed, attr)   	ted_copy(sed, NULL, attr, B_ATTR)
#define ted_BlockPaste(sed, buf)	ted_insert(sed, buf)

#define	ted_GetWrapWidth(sed)  		((sed_GetMenu(sed))->textbuf->width)
#define	ted_SetWrapWidth(sed, w)  	ted_reset(sed, w, 0)
#define	ted_GetTabSize(sed) 	 	((sed_GetMenu(sed))->textbuf->tab_size)
#define	ted_SetTabSize(sed, t)  	ted_reset(sed, t, 1)

#define ted_GetRefresh(sed)			((sed_GetMenu(sed))->textbuf->refresh)
#define ted_SetRefresh(sed, mode)	((sed_GetMenu(sed))->textbuf->refresh = (mode))
#define ted_GetInsert(sed)			((sed_GetMenu(sed))->textbuf->insert)
#define ted_SetInsert(sed, mode)	((sed_GetMenu(sed))->textbuf->insert = (mode))
#define ted_GetTabChar(sed)	 		((sed_GetMenu(sed))->textbuf->tab_char)
#define ted_SetTabChar(sed, chr) 	((sed_GetMenu(sed))->textbuf->tab_char = (chr))
#define ted_GetNewLineChar(sed) 	((sed_GetMenu(sed))->textbuf->newline_char)
#define ted_SetNewLineChar(sed, chr)	((sed_GetMenu(sed))->textbuf->newline_char = (chr))

#define ted_GetMaxSize(sed) 		(((sed_GetMenu(sed))->textbuf->limit ) ? (sed_GetMenu(sed))->textbuf->max_size : 0L)
#define ted_SetMaxSize(sed, size)  	tb_SetMaxSize((sed_GetMenu(sed))->textbuf, size)
#define ted_GetSize(sed) 			tb_GetSize((sed_GetMenu(sed))->textbuf)
#define ted_GetMoveMethod(sed) 		((sed_getxd(sed))->move)
#define ted_GetLineLen(sed)			ted_line(sed, TED_LEN)
#define ted_IsHardLine(sed)			ted_line(sed, TED_HARD)

#define is_whitespace(c)  ((c == ' ' || c == '\t' || c == '\n') ? 1 : 0)



