/*
	tedit.c  

	% commmon routines used in text editting

	C-scape 3.1
	Copyright (c) 1988 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 6/17/88 jdc	created
	 9/22/88 jmd	Fixed ted_reset... now calls sed_Update
	12/18/88 jdc	added check before BDM_SCROLL in ted_DeleteString

	 2/08/89 jmd	added args
	 3/24/89 jmd	added sed_ macros
	 4/06/89 jmd	made this match teddecl (int to boolean)
	 5/19/89 jdc	removed cursor from ted_DeleteString
	05/23/89 jdc	made tb_setcursor menu_setcursor
	05/24/89 jdc	added clipping to sed_Paint, now takes menu coords
	05/25/89 jdc	removed sed_Paint, now uses sd_paintbox
	05/28/89 jdc	fixed ted_PageDown for LNF
	 6/26/89 jmd	removed unused yoffset from ted_PageDown
	 7/03/89 jdc	fixed ted_reset
	 7/21/89 jdc	fixed ted_reset again, off_text case
	 7/22/89 jdc	moved ted_SetMoveMethod to tedsetmv.c
	 8/14/89 jmd	Added call to disp_evcheck() macro
	 9/08/89 jmd    Added LNF stuff
*/

#ifdef LNF
#include "lnf.h"
#include "teddecl.h"
#include "tbpriv.h"
#else
#include "ted.h"
#endif

void ted_StartWorking(sed)
	sed_type sed;
{
	if (sed_GetMoveMethod(sed) == default_move) {
		sed_SetMove(sed, ted_Follow);
	}

	(sed_GetMenu(sed))->vwidth = ted_GetWrapWidth(sed);

	ted_GotoPosition(sed, sed_GetCRow(sed), sed_GetCCol(sed));
	ted_SetCursor(sed, sed_GetCRow(sed), sed_GetCCol(sed));
	win_ShowCursor(sed);
}

void ted_GetPosition(sed, row, col)
	sed_type sed;
	int *row, *col;
{
	*row = sed_GetCRow(sed);
	*col = sed_GetCCol(sed);
}

boolean ted_LeftChar(sed, tab_mode)
	sed_type sed;
	int tab_mode;
/*
	if tab_mode == TED_TABJUMP then move by whole tabs,
	else internal to tab.
*/
{
    int row, col;
	boolean ret;
	tb_type tb;
		
	ted_GetPosition(sed, &row, &col);
	if ((ret = ted_GotoPosition(sed, row, --col)) == TRUE
		&& tab_mode == TED_TABJUMP) {

		tb = sed_GetTextbuf(sed);
		while (row == tb_GetRow(tb) && col < tb->exp_len 
			&& col != tb_GetCol(tb)) {

			ted_GotoPosition(sed, row, --col);
		}
	}
	return(ret);
}	
			
boolean ted_RightChar(sed, tab_mode)
	sed_type sed;
	int tab_mode;
/*
	if tab_mode == TED_TABJUMP then move by whole tabs,
	else internal to tab.
*/
{
    int row, col;
	boolean ret;
	tb_type tb;
		
	ted_GetPosition(sed, &row, &col);
	if ((ret = ted_GotoPosition(sed, row, ++col)) == TRUE
		&& tab_mode == TED_TABJUMP) {

		tb = sed_GetTextbuf(sed);
		while (row == tb_GetRow(tb) && col < tb->exp_len
			&& col != tb_GetCol(tb)) {

			ted_GotoPosition(sed, row, ++col);
		}
	}
	return(ret);
}	

boolean	ted_GoEnd(sed)
	sed_type sed;
{
	tb_type tb;
	int row, col;

	ted_GetPosition(sed, &row, &col);
	tb = sed_GetTextbuf(sed);

	if (row > tb_GetRow(tb)) {
		col = 0;
	}
	else {
		col = tb->exp_len - 1;
	}
	return(ted_GotoPosition(sed, row, col));
}

void ted_PageUp(sed)
	sed_type sed;
{
	int row, col, yoffset;

	ted_GetPosition(sed, &row, &col);

	if (sed_GetYoffset(sed) == 0) {
		row = 0;
	}
	else {
		yoffset = (sed_GetYoffset(sed) > sed_GetHeight(sed)) ?
			sed_GetHeight(sed) : sed_GetYoffset(sed);

		row -= yoffset;
		if (ted_GetMark(sed) != TED_NOMARK) {

			/* cheat to prevent marking flash */
			if (!((sed_GetTextbuf(sed))->m_stop)) {
				(sed_GetTextbuf(sed))->markbox.cleat_row -= yoffset;
			}
		}
		sd_scroll(sed, -yoffset, 0, FALSE);
	}
	ted_GotoPosition(sed, row, col);
}

void ted_PageDown(sed)
	sed_type sed;
{
	int refresh;
	int row, col;

	ted_GetPosition(sed, &row, &col);

	row += sed_GetHeight(sed);
	refresh = ted_GetRefresh(sed);
	ted_SetRefresh(sed, TED_NOREFRESH);

	if (!ted_GotoPosition(sed, row, col)) {

 		ted_SetRefresh(sed, refresh);
		ted_GoBottom(sed);
	}
	else {
		sd_scroll(sed, sed_GetHeight(sed), 0, FALSE);

		if (ted_SetRefresh(sed, refresh) == TED_REFRESH) {
			sed_Update(sed);
		}
		ted_SetCursor(sed, row, col);
	}
}

void ted_GoBottom(sed)
	sed_type sed;
/*
	moves the cursor to the last row 
*/
{
	int row;
	tb_type tb;

	tb = sed_GetTextbuf(sed);	
	row = sed_GetCRow(sed);

	if (!tb_FindPosition(tb, row, 0)) {
		/* find last row backwards */
		while (!tb_FindPosition(tb, row, 0)) {
			row--;
		}
	}
	else {
		/* find last row forwards */
		while (tb_FindPosition(tb, row, 0)) {
			row++;
		}
		row--;
	}
	ted_GotoPosition(sed, row, tb->exp_len - 1);
}

void ted_reset(sed, i, mode)
	sed_type sed;
	int i, mode;
/* 
	resets wrap width (mode == 0) or tab size (mode == 1), and hints
*/
{
	tb_type tb;
	int row, col, off_text = FALSE;
	long cursor;

	tb = sed_GetTextbuf(sed);	
	ted_GetPosition(sed, &row, &col);
	cursor = tb->cursor;
	
	if (!tb_FindPosition(tb, row, col)) {
		off_text = TRUE;
	}
	if (mode == 0) {
		tb_SetWrapWidth(tb, i);
		(sed_GetMenu(sed))->vwidth = i;
	 	/* Send Scroll message to the border */
		sed_SendBorderMsg(sed, BDM_SCROLL, NULL, NULL);
	}
	else {
		tb_SetTabSize(tb, i);
	}
	tb_Rewind(tb);
	if (off_text) {
		tb_FindPosition(tb, row, col);
	}
	else {
		tb->cursor = cursor;
		menu_setcursor(sed_GetMenu(sed));
		if (ted_GetRefresh(sed)) {
			ted_GotoPosition(sed, tb_GetRow(tb), tb_GetCol(tb));
		}
	}
}

void ted_SetCursor(sed, row, col)
	sed_type sed;
	int row, col;
/* 
	sets the window cursor from the sed's (menu!) cursor position 
*/
{
	tb_type tb;
	int lrow;
	
	lrow = sed_GetCRow(sed);	
	sed_SetTrow(sed, row);
	sed_SetTcol(sed, col);

#ifdef LNF
	if (lnf_GetMode(lnf) == MODE_NORM) {
		lnf_GotoField(lnf, sed);
		lnf_ShowPosition(lnf, sed);
	}
#endif

	tb = sed_GetTextbuf(sed);	
	if (ted_GetMark(sed) != TED_NOMARK) {
		if (!(tb->m_stop)) {
			tb->markbox.cleat_row = row;
			tb->markbox.cleat_col = col;
			tb->cleat = tb->cursor;
		}
	}
	if (ted_GetRefresh(sed)) {
		ted_Scroll(sed);
		win_SetCursorPos(sed, row - sed_GetYoffset(sed), col - sed_GetXoffset(sed));
		if (!(tb->m_stop) && (tb->mark == TED_MARK || tb->mark == TED_COLMARK)) {
			if (oabs(row - lrow) > oabs(row - tb->markbox.anchor_row)) {
				sed_RepaintRows(sed, (row < lrow) ? row:lrow, (row > lrow) ? row:lrow);
			}
			else if (row < tb->markbox.anchor_row) {
				sed_RepaintRows(sed, (row < lrow) ? row : lrow, tb->markbox.anchor_row);
			}
			else {
				sed_RepaintRows(sed, tb->markbox.anchor_row, (row > lrow) ? row : lrow);
			}
		}
	}
}

#define HORZ_JUMP 5

boolean ted_Scroll(sed)
	sed_type sed;
/* 
	checks whether the cursor is outside the window and scrolls appropriately 
*/
{
 	int offset, ret = FALSE, row, col;
	int deltaY = 0, deltaX = 0;

	row = sed_GetCRow(sed) - sed_GetYoffset(sed);
	col = sed_GetCCol(sed) - sed_GetXoffset(sed);

	if ((offset = row) < 0) {
		deltaY = offset;
		ret = TRUE;
	}		
	else if ((offset = row - (sed_GetHeight(sed) - 1)) > 0){
		deltaY = offset;
		ret = TRUE;
	}

	if ((offset = col) < 0) {
		deltaX = (offset/HORZ_JUMP - 1) * HORZ_JUMP;
		deltaX = (deltaX + sed_GetXoffset(sed) < 0) ? (-sed_GetXoffset(sed)) : deltaX;
		ret = TRUE;
	}
	else if ((offset = col - (sed_GetWidth(sed) - 1)) > 0){
		deltaX = (offset/HORZ_JUMP + 1) * HORZ_JUMP;
		ret = TRUE;
	}
	if (ret) {
		row += sed_GetYoffset(sed);
		col += sed_GetXoffset(sed);
		sd_scroll(sed, deltaY, deltaX, FALSE);				
	}												

	return(ret);
}

boolean ted_AddChar(sed, c)
	sed_type sed;
	char c;
{
	return(ted_AddString(sed, &c, 1));
}

boolean ted_AddString(sed, s, len)
	sed_type sed;
	char *s;
	unsigned int len;
/* 
	places s at the cursor, resets hints, and refreshes display 
*/
{
	tb_type tb;
	int		row, col, refresh_row;
	int 	ret;

	ted_GetPosition(sed, &row, &col);
	tb = sed_GetTextbuf(sed);	

	if ((refresh_row = tb_GetRow(tb)) > row - 1) {
		refresh_row = (row > 0) ? row - 1 : 0;
	}
	if (row == tb_GetRow(tb) && col != tb->col && col < tb->exp_len) {
		/* in middle of tab */
		if (!menu_Addc(sed_GetMenu(sed), row, tb->col, ' ', col - tb->col)) {
			ret = FALSE;
		}
	}		
	ret = menu_Adds(sed_GetMenu(sed), row, col, s, len);

	ted_GotoPosition(sed, tb_GetRow(tb), tb_GetCol(tb));
	if (ted_GetRefresh(sed)) {

		if (!disp_EvCheck()) {
			/* no kb_Check support on this system, paint everything */
			sed_RepaintBelow(sed, refresh_row);
		}
		else {
			/* if kb_Check works, use it.
			   Paint less when there are pending keystrokes 
			*/
			if (kb_Check()) {
				sed_RepaintRow(sed, row, col, sed_GetWidth(sed));
			}
			else {
				sed_RepaintBelow(sed, refresh_row);
			}
		}

		if (tb_GetRow(tb) > row) {
			sed_SendBorderMsg(sed, BDM_SCROLL, NULL, NULL);
		}
	}
	tb->xcol = sed_GetCCol(sed);
	return(ret);
}

char ted_DeleteString(sed, row, col, len)
	sed_type sed;
	int row, col;
	unsigned int len;
/* 
	deletes the string at the row, col.
	resets hints, and refreshes display 
*/
{
	tb_type tb;
	int spaces, rowcount;
	char oldchr[2];

	tb = sed_GetTextbuf(sed);	

	if (!tb_FindPosition(tb, row, col)) {
		return('\0');
	}
	ted_GetString(sed, oldchr, 1);	/* remember the first deleted char */

	if ((spaces = col - (tb->exp_len - 1)) > 0) {
		menu_Addc(sed_GetMenu(sed), row, tb->len, ' ', spaces);
		tb_SetLineLen(tb);
	}				
	rowcount = (sed_GetMenu(sed))->rowcount;
	if (menu_strdel(sed_GetMenu(sed), row, col, (long)len) == 0L) {
		*oldchr = '\0';
	}
	ted_GotoPosition(sed, tb_GetRow(tb), tb_GetCol(tb));
	if (ted_GetRefresh(sed)) {
		sed_RepaintBelow(sed, (row <= 1) ? 0 : row - 1);

		/* size change */
		if (rowcount != (sed_GetMenu(sed))->rowcount) {
			sed_SendBorderMsg(sed, BDM_SCROLL, NULL, NULL);
		}
	}
	return(*oldchr);
}

